// Tencent is pleased to support the open source community by making ncnn available.
//
// Copyright (C) 2021 THL A29 Limited, a Tencent company. All rights reserved.
//
// Licensed under the BSD 3-Clause License (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
//
// https://opensource.org/licenses/BSD-3-Clause
//
// Unless required by applicable law or agreed to in writing, software distributed
// under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
// CONDITIONS OF ANY KIND, either express or implied. See the License for the
// specific language governing permissions and limitations under the License.

#ifndef NDKCAMERA_H
#define NDKCAMERA_H

#include <camera/NdkCameraDevice.h>
#include <camera/NdkCameraManager.h>
#include <camera/NdkCameraMetadata.h>
#include <media/NdkImageReader.h>

#include <opencv2/core/core.hpp>
#include "Camera2Helper.h"

/**
 * Range of Camera Exposure Time:
 *     Camera's capability range have a very long range which may be disturbing
 *     on camera. For this sample purpose, clamp to a range showing visible
 *     video on preview: 100000ns ~ 250000000ns
 */
static const uint64_t kMinExposureTime = static_cast<uint64_t>(1000000);
static const uint64_t kMaxExposureTime = static_cast<uint64_t>(250000000);


class NdkCamera
{
public:


    NdkCamera(int32_t width, int32_t height);
    virtual ~NdkCamera();

    // facing 0=front 1=back
    int open(const char* cameraId);
    void close();

    void onImageAvailable(AImageReader* reader);
    virtual bool on_image(const cv::Mat& rgb);
    virtual void on_error(const std::string& msg);

    virtual void on_image(const unsigned char* nv21, int nv21_width, int nv21_height);

    void onCaptureCompleted(ACameraCaptureSession* session, ACaptureRequest* request, const ACameraMetadata* result);


public:
    int camera_facing;
    int camera_orientation;
    bool m_firstFrame;
    int32_t mWidth;
    int32_t mHeight;
    std::string mCameraId;

    bool hdrSupported;
    bool nightModeSupported;
    bool nightPortraitModeSupported;
    bool afSupported;
    bool awbSupported;
    // int64_t exposureTime_;
    RangeValue<int64_t> exposureRange;
    // int32_t sensitivity_;
    RangeValue<int32_t> sensitivityRange;
    RangeValue<int32_t> aeCompensationRange;



private:
    ACameraManager* camera_manager;
    ACameraDevice* camera_device;
    AImageReader* image_reader;
    ANativeWindow* image_reader_surface;
    ACameraOutputTarget* image_reader_target;
    ACaptureRequest* capture_request;
    ACaptureSessionOutputContainer* capture_session_output_container;
    ACaptureSessionOutput* capture_session_output;
    ACameraCaptureSession* capture_session;
};

#endif // NDKCAMERA_H
